"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolicyManager = void 0;
const tslib_1 = require("tslib");
const core_1 = require("./helpers/core");
const policy_1 = require("./interfaces/policy");
class PolicyManager extends core_1.CoreHelper {
    constructor(storage) {
        super();
        this.storage = storage;
    }
    setStorage(storage) {
        this.storage = storage;
        return this;
    }
    attach(rawPrincipal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
            return this.storage.add(principal, rawPolicies);
        });
    }
    reset(rawPrincipal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
            return rawPolicies
                ? this.storage.save(principal, rawPolicies)
                : this.storage.purge(principal);
        });
    }
    retrieve(rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
            const rawPolicies = yield this.storage.fetch(principal);
            return rawPolicies.map((rawPolicy) => typeof rawPolicy === 'string' ? core_1.CoreHelper.decode(rawPolicy) : rawPolicy);
        });
    }
    retrieveBySid(sid, rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
            const rawPolicies = yield this.storage.fetchBySid(sid, principal);
            return rawPolicies.map((rawPolicy) => typeof rawPolicy === 'string' ? core_1.CoreHelper.decode(rawPolicy) : rawPolicy);
        });
    }
    upsertBySid(sid, rawPrincipal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
            return this.storage.saveBySid(sid, principal, rawPolicies);
        });
    }
    grant(rawAction, rawPrincipal, rawResource = core_1.CoreHelper.ANY, effect = policy_1.Effect.ALLOW, sid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resource = (this.normalizeDynamicObject(rawResource));
            const principal = (this.normalizeDynamicObject(rawPrincipal));
            const policy = {
                [policy_1.SID]: sid,
                [policy_1.EFFECT]: effect,
                [policy_1.ACTION]: rawAction,
                [policy_1.RESOURCE]: resource,
                [policy_1.PRINCIPAL]: principal,
            };
            return this.attach(rawPrincipal, [policy]);
        });
    }
}
exports.PolicyManager = PolicyManager;
