"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SequelizeStorage = void 0;
const tslib_1 = require("tslib");
class SequelizeStorage {
    constructor(storageRepository) {
        this.storageRepository = storageRepository;
        this.readonly = false;
    }
    fetch(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.repository.findByPrincipal(principal);
        });
    }
    fetchBySid(sid, principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.repository.findByPrincipal(principal, sid);
        });
    }
    save(principal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.repository.savePrincipalPolicies(principal, rawPolicies, false);
        });
    }
    add(principal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.repository.savePrincipalPolicies(principal, rawPolicies, true);
        });
    }
    saveBySid(sid, principal, rawPolicies) {
        return this.repository.savePrincipalPolicies(principal, rawPolicies, false, sid);
    }
    purge(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.repository.destroyByPrincipal(principal);
        });
    }
    get repository() {
        if (!this.isStorageModel(this.storageRepository)) {
            throw new TypeError('Policies repository must extend Sequelize PoliciesStorage model');
        }
        return this.storageRepository;
    }
    isStorageModel(x) {
        return (typeof x === 'function' &&
            typeof x.findByPrincipal === 'function' &&
            typeof x.savePrincipalPolicies === 'function' &&
            typeof x.destroyByPrincipal === 'function');
    }
}
exports.SequelizeStorage = SequelizeStorage;
